==> fred_inquisitor-1.0.0rc4-py3-none-any.whl: inquisitor/contact.py <==
"""Contact endpoints."""
from typing import Any, cast

from fastapi import APIRouter, Depends, HTTPException, status
from fred_types import ContactId
from grill import Logger
from regal import (
    Contact,
    ContactClient,
    DomainClient,
    DomainContactRole,
    Registrar,
    RegistrarClient,
)
from regal.exceptions import ContactDoesNotExist, RegistrarDoesNotExist
from verifier import VerificationClient, VerificationStatus
from verifier.exceptions import IncorrectStatus

from .auth import get_contact_id
from .constants import COMMON_RESPONSES, LogEntryType, LoggerResult, LogReferenceType, MessageType
from .models import Error, PaginatedList, Pagination
from .settings import (
    get_contact_client,
    get_domain_client,
    get_logger,
    get_registrar_client,
    get_verification_client,
)
from .utils import get_source_ip

router = APIRouter(prefix="/contact")

RESPONSES: dict[int | str, dict[str, Any]] = {
    **COMMON_RESPONSES,
    404: {"description": "Not found", "model": Error},
}


@router.get("/", responses=RESPONSES)
async def get(
    contact_id: ContactId = Depends(get_contact_id),
    contact_client: ContactClient = Depends(get_contact_client),
) -> Contact:
    """Get current contact info."""
    try:
        return await contact_client.get_contact_info(contact_id)
    except ContactDoesNotExist:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found")


@router.get("/domains/", responses=RESPONSES)
async def get_domains(
    contact_id: ContactId = Depends(get_contact_id),
    domain_client: DomainClient = Depends(get_domain_client),
    page_size: int | None = None,
    page_token: str | None = None,
) -> PaginatedList[str]:
    """Get list of undeleted domain names owned by contact.

    The list is ordered by FQDN.
    The list can be pagineted using `page_size` and `page_tokens` query params.
    """
    try:
        domains = await domain_client.get_domains_by_contact(
            contact_id,
            roles={DomainContactRole.HOLDER},
            include_deleted=False,
            order_by="fqdn",
            page_size=page_size,
            page_token=page_token,
        )
        return PaginatedList[str](
            results=[result["domain"].fqdn for result in domains.get("domains", [])],
            pagination=Pagination(**domains["pagination"]) if domains.get("pagination") else None,
        )
    except ContactDoesNotExist:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found")


class RegistrarPublicInfo(Registrar):
    """Registrar public info."""

    class Config:
        fields = {
            "registrar_id": {"include": True},
            "registrar_handle": {"include": True},
            "name": {"include": True},
            "organization": {"include": True},
            "place": {"include": True},
            "url": {"include": True},
        }


@router.get("/registrar/", responses=RESPONSES)
async def get_registrar(
    contact_id: ContactId = Depends(get_contact_id),
    contact_client: ContactClient = Depends(get_contact_client),
    registrar_client: RegistrarClient = Depends(get_registrar_client),
) -> RegistrarPublicInfo:
    """Get contact registrar public info."""
    try:
        contact = await contact_client.get_contact_info(contact_id)
        return cast(
            RegistrarPublicInfo,
            await registrar_client.get_registrar_info(cast(str, contact.sponsoring_registrar)),
        )
    except ContactDoesNotExist:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found")
    except RegistrarDoesNotExist:
        raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Registrar not found")


@router.post("/resend_initial/", responses={404: {"description": "Not found", "model": Error}})
async def resend_initial_message(
    contact_handle: str,
    source_ip: str | None = Depends(get_source_ip),
    logger: Logger = Depends(get_logger),
    contact_client: ContactClient = Depends(get_contact_client),
    verification_client: VerificationClient = Depends(get_verification_client),
) -> None:
    """Resend initial message.

    If there is no such contact, return 404 Error.

    If there is such contact and has opened verification, resend
    the initial message.

    For security reasons, endpoint does not provide information
    whether there is opened verification for contact or not.
    If there is none, this endpoint is NOOP.
    """
    log_entry = logger.create(
        LogEntryType.RESEND_MESSAGE,
        source_ip=source_ip,
        properties={"contact_handle": contact_handle, "type": MessageType.INITIAL.value},
    )
    with log_entry:
        try:
            log_entry.properties["message_sent"] = str(False)
            contact_id = await contact_client.get_contact_id(contact_handle)
            log_entry.references[LogReferenceType.CONTACT] = contact_id
            opened_statuses = set(VerificationStatus) - {
                VerificationStatus.SUCCEEDED,
                VerificationStatus.FAILED,
                VerificationStatus.CANCELED,
            }
            verifications = await verification_client.list(
                contact_id=contact_id, statuses=opened_statuses
            )
            if verifications.results:
                verification = verifications.results[0]
                log_entry.references[LogReferenceType.VERIFICATION] = verification.verification_id
                await verification_client.resend_message(
                    verification.verification_id, message_type=MessageType.INITIAL
                )
                log_entry.properties["message_sent"] = str(True)
            log_entry.result = LoggerResult.SUCCESS
        except ContactDoesNotExist:
            log_entry.properties["error"] = "Contact not found"
            log_entry.result = LoggerResult.FAIL
            raise HTTPException(status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found")
        except IncorrectStatus:
            log_entry.properties["error"] = "Incorrect verification status"
            log_entry.result = LoggerResult.FAIL

==> fred_inquisitor-1.3.0-py3-none-any.whl: inquisitor/contact.py <==
"""Contact endpoints."""

from typing import Any, cast

from fastapi import APIRouter, Depends, HTTPException, status
from fred_types import ContactId, RegistrarId
from grill import Logger
from pydantic import BaseModel
from regal import (
    Address,
    Contact,
    ContactClient,
    DomainClient,
    DomainContactRole,
    RegistrarClient,
)
from regal.exceptions import ContactDoesNotExist, RegistrarDoesNotExist
from verifier import VerificationClient, VerificationStatus
from verifier.exceptions import IncorrectStatus

from .constants import COMMON_RESPONSES, LogEntryType, LoggerResult, LogReferenceType, MessageType
from .models import Error, PaginatedList, Pagination
from .settings import (
    get_contact_client,
    get_domain_client,
    get_logger,
    get_registrar_client,
    get_verification_client,
)
from .token import get_contact_id
from .utils import get_source_ip

router = APIRouter(prefix="/contact")

RESPONSES: dict[int | str, dict[str, Any]] = {
    **COMMON_RESPONSES,
    404: {"description": "Not found", "model": Error},
}


@router.get("/", responses=RESPONSES)
async def get(
    contact_id: ContactId = Depends(get_contact_id),
    contact_client: ContactClient = Depends(get_contact_client),
) -> Contact:
    """Get current contact info."""
    try:
        return await contact_client.get_contact_info(contact_id)
    except ContactDoesNotExist:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found"
        ) from None


@router.get("/domains/", responses=RESPONSES)
async def get_domains(
    contact_id: ContactId = Depends(get_contact_id),
    domain_client: DomainClient = Depends(get_domain_client),
    page_size: int | None = None,
    page_token: str | None = None,
) -> PaginatedList[str]:
    """Get list of undeleted domain names owned by contact.

    The list is ordered by FQDN.
    The list can be pagineted using `page_size` and `page_tokens` query params.
    """
    try:
        domains = await domain_client.get_domains_by_contact(
            contact_id,
            roles={DomainContactRole.HOLDER},
            include_deleted=False,
            order_by="fqdn",
            page_size=page_size,
            page_token=page_token,
        )
        return PaginatedList[str](
            results=[result["domain"].fqdn for result in domains.get("domains", [])],
            pagination=Pagination(**domains["pagination"]) if domains.get("pagination") else None,
        )
    except ContactDoesNotExist:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found"
        ) from None


class RegistrarPublicInfo(BaseModel):
    """Registrar public info."""

    registrar_id: RegistrarId
    registrar_handle: str
    name: str
    organization: str = ""
    place: Address | None = None
    url: str | None = None


@router.get("/registrar/", responses=RESPONSES)
async def get_registrar(
    contact_id: ContactId = Depends(get_contact_id),
    contact_client: ContactClient = Depends(get_contact_client),
    registrar_client: RegistrarClient = Depends(get_registrar_client),
) -> RegistrarPublicInfo:
    """Get contact registrar public info."""
    try:
        contact = await contact_client.get_contact_info(contact_id)
        return cast(
            RegistrarPublicInfo,
            await registrar_client.get_registrar_info(cast(str, contact.sponsoring_registrar)),
        )
    except ContactDoesNotExist:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found"
        ) from None
    except RegistrarDoesNotExist:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND, detail="Registrar not found"
        ) from None


@router.post("/resend_initial/", responses={404: {"description": "Not found", "model": Error}})
async def resend_initial_message(
    contact_handle: str,
    source_ip: str | None = Depends(get_source_ip),
    logger: Logger = Depends(get_logger),
    contact_client: ContactClient = Depends(get_contact_client),
    verification_client: VerificationClient = Depends(get_verification_client),
) -> None:
    """Resend initial message.

    If there is no such contact, return 404 Error.

    If there is such contact and has opened verification, resend
    the initial message.

    For security reasons, endpoint does not provide information
    whether there is opened verification for contact or not.
    If there is none, this endpoint is NOOP.
    """
    log_entry = logger.create(
        LogEntryType.RESEND_MESSAGE,
        source_ip=source_ip,
        properties={"contact_handle": contact_handle, "type": MessageType.INITIAL.value},
    )
    with log_entry:
        try:
            log_entry.properties["message_sent"] = str(False)
            contact_id = await contact_client.get_contact_id(contact_handle)
            log_entry.references[LogReferenceType.CONTACT] = contact_id
            opened_statuses = set(VerificationStatus) - {
                VerificationStatus.SUCCEEDED,
                VerificationStatus.FAILED,
                VerificationStatus.CANCELED,
                VerificationStatus.MANUAL,
            }
            verifications = await verification_client.list(
                contact_id=contact_id, statuses=opened_statuses
            )
            if verifications.results:
                verification = verifications.results[0]
                log_entry.references[LogReferenceType.VERIFICATION] = verification.verification_id
                await verification_client.resend_message(
                    verification.verification_id, message_type=MessageType.INITIAL
                )
                log_entry.properties["message_sent"] = str(True)
            log_entry.result = LoggerResult.SUCCESS
        except ContactDoesNotExist:
            log_entry.properties["error"] = "Contact not found"
            log_entry.result = LoggerResult.FAIL
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND, detail="Contact not found"
            ) from None
        except IncorrectStatus:
            log_entry.properties["error"] = "Incorrect verification status"
            log_entry.result = LoggerResult.FAIL
